import { ListOperationOptions } from '@rokoli/bnb'
import { defineStore } from 'pinia'
import { APIListUnpaginated, createExtendableAPI } from '@rokoli/bnb/drf'
import { steeringAuthInit } from '@/stores/auth'
import { createSteeringURL } from '@/api'
import { ProgramEntry } from '@/types'
import { useQuery } from '@/util'
import { MaybeRefOrGetter, readonly, ref, toValue, watch } from 'vue'

export const useProgramStore = defineStore('program', () => {
  const endpoint = createSteeringURL.prefix('program', 'basic')
  const { api } = createExtendableAPI<ProgramEntry>(endpoint, steeringAuthInit)
  const { list: _list } = APIListUnpaginated(api)

  return {
    list: (options: ListOperationOptions | undefined = undefined) => {
      return _list({ ...(options ?? {}), noStoreCommit: true })
    },
  }
})

interface UseProgramOptions {
  start: MaybeRefOrGetter<string | Date>
  end: MaybeRefOrGetter<string | Date>
}

export function useProgramSlots(options: UseProgramOptions) {
  const programStore = useProgramStore()
  const error = ref<Error | undefined>()
  const isLoading = ref<boolean>(false)
  let controller: AbortController | undefined = undefined
  const program = ref<ProgramEntry[]>([])

  const query = useQuery(() => ({
    start: toValue(options.start),
    end: toValue(options.end),
    includeVirtual: true,
  }))

  async function updateProgram() {
    if (controller) controller.abort('date changed')
    error.value = undefined
    controller = new AbortController()
    isLoading.value = true
    try {
      program.value = await programStore.list({
        query: query.value,
        requestInit: { signal: controller.signal },
      })
    } finally {
      isLoading.value = false
      controller = undefined
    }
  }

  watch(query, updateProgram, { immediate: true })

  return {
    program: readonly(program),
    error: readonly(error),
    isLoading: readonly(isLoading),
    reload: updateProgram,
  }
}
